from pathlib import Path
from typing import Union

from pytest_zebrunner.api.client import ZebrunnerAPI
from pytest_zebrunner.api.models import ArtifactReferenceModel, LabelModel
from pytest_zebrunner.context import zebrunner_context


def attach_test_screenshot(path: Union[str, Path]) -> None:
    """
    Attach a screenshot to the test in scope of Zebrunner Reporting in order to track it.

    Args:
        path (Union[str, Path]): Path to identify image location in directory structure.
    """
    api = ZebrunnerAPI(zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token)
    if zebrunner_context.test_is_active:
        api.send_screenshot(zebrunner_context.test_run_id, zebrunner_context.test_id, path)


def attach_test_artifact(path: Union[str, Path]) -> None:
    """
    Attach an artifact to the test in scope of Zebrunner Reporting in order to track it.
    The display name provided must contain the file extension that reflects file's content.

    Args:
        path (Union[str, Path]): Path to identify artifact location in directory structure.
    """
    api = ZebrunnerAPI(zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token)
    if zebrunner_context.test_is_active:
        api.send_artifact(path, zebrunner_context.test_run_id, zebrunner_context.test_id)


def attach_test_run_artifact(path: Union[str, Path]) -> None:
    """
    Attach an artifact to the test_run in scope of Zebrunner Reporting in order to track it.
    The display name provided must contain the file extension that reflects file's content.

    Args:
        path (Union[str, Path]): Path to identify artifact location in directory structure.
    """
    api = ZebrunnerAPI(zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token)
    if zebrunner_context.test_run_is_active:
        api.send_artifact(path, zebrunner_context.test_run_id)


def attach_test_artifact_reference(name: str, ref: str) -> None:
    """
    Attach an artifact reference to the test in scope of Zebrunner Reporting in order to track it.

    Args:
        name (str): Arbitrary name to identify the reference.
        ref (str): Reference to the artifact.
    """
    api = ZebrunnerAPI(zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token)
    if zebrunner_context.test_is_active:
        api.send_artifact_references(
            [ArtifactReferenceModel(name=name, value=ref)], zebrunner_context.test_run_id, zebrunner_context.test_id
        )


def attach_test_run_artifact_reference(name: str, ref: str) -> None:
    """
    Attach an artifact reference to the test_run in scope of Zebrunner Reporting in order to track it.

    Args:
        name (str): Arbitrary name to identify the reference
        ref (str): Reference to the artifact.
    """
    api = ZebrunnerAPI(zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token)
    if zebrunner_context.test_run_is_active:
        api.send_artifact_references([ArtifactReferenceModel(name=name, value=ref)], zebrunner_context.test_run_id)


def attach_test_label(name: str, value: str) -> None:
    """
    Attach meta data to the test as key-value pair, in scope of Zebrunner Reporting in order to track it.

    Args:
        name (str): Key of key-value pair.
        value (str): Value of key-value pair.
    """
    api = ZebrunnerAPI(zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token)
    if zebrunner_context.test_is_active:
        api.send_labels([LabelModel(key=name, value=value)], zebrunner_context.test_run_id, zebrunner_context.test_id)


def attach_test_run_label(name: str, value: str) -> None:
    """
    Attach meta data to the test_run as key-value pair, in scope of Zebrunner Reporting in order to track it.

    Args:
        name (str): Key of key-value pair.
        value (str): Value of key-value pair.
    """
    api = ZebrunnerAPI(zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token)
    if zebrunner_context.test_run_is_active:
        api.send_labels([LabelModel(key=name, value=value)], zebrunner_context.test_run_id)



import logging
import time
from datetime import datetime, timedelta
from logging import LogRecord, StreamHandler
from typing import List

import httpx

from pytest_zebrunner.api.client import ZebrunnerAPI
from pytest_zebrunner.api.models import LogRecordModel
from pytest_zebrunner.context import zebrunner_context


class ZebrunnerHandler(StreamHandler):
    """
    A class that inherit from StreamHandler useful for recording logs.

    Attributes:
        logs (List[LorRecordModel]): List of logs to be handled.
    """

    logs: List[LogRecordModel] = []

    def __init__(self) -> None:
        super().__init__()
        self.api = ZebrunnerAPI(
            zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token
        )
        self.last_push = datetime.utcnow()

    def emit(self, record: LogRecord) -> None:
        """
        Try to send logs to test_run_id if the last attempt was more than a second ago. If not, and test is active,
        adds a new log to the list.

        Args:
            record (LogRecord): The log to be recorded.
        """
        if datetime.utcnow() - self.last_push >= timedelta(seconds=1):
            self.push_logs()

        if zebrunner_context.test_is_active:
            self.logs.append(
                LogRecordModel(
                    test_id=str(zebrunner_context.test_id),
                    timestamp=str(round(time.time() * 1000)),
                    level=record.levelname,
                    message=str(record.msg),
                )
            )

    def push_logs(self) -> None:
        """
        Updates last_push datetime, resets logs list and send the to Zebrunner API
        for reporting if test_run_id is active.

        """
        try:
            if zebrunner_context.test_run_id and zebrunner_context.settings.send_logs:
                self.api.send_logs(zebrunner_context.test_run_id, self.logs)
        except httpx.HTTPError as e:
            logging.error("Failed to send logs to zebrunner", exc_info=e)
        finally:
            self.logs = []
            self.last_push = datetime.utcnow()



import logging
from typing import List, Optional

import pytest
from _pytest._code.code import ExceptionChainRepr, ReprExceptionInfo
from _pytest.nodes import Item
from _pytest.reports import TestReport

from pytest_zebrunner.api.client import ZebrunnerAPI
from pytest_zebrunner.api.models import (
    ArtifactReferenceModel,
    CorrelationDataModel,
    FinishTestModel,
    FinishTestSessionModel,
    MilestoneModel,
    NotificationsType,
    NotificationTargetModel,
    StartTestModel,
    StartTestRunModel,
    StartTestSessionModel,
    TestRunConfigModel,
    TestStatus,
)
from pytest_zebrunner.ci_loaders import resolve_ci_context
from pytest_zebrunner.context import Test, TestRun, zebrunner_context
from pytest_zebrunner.zebrunner_logging import ZebrunnerHandler

logger = logging.getLogger(__name__)


class ReportingService:
    def __init__(self) -> None:
        self.api = ZebrunnerAPI(
            zebrunner_context.settings.server.hostname, zebrunner_context.settings.server.access_token
        )

    def authorize(self) -> None:
        if not self.api.authenticated:
            self.api.auth()

    def get_notification_configurations(self) -> Optional[List[NotificationTargetModel]]:
        settings = zebrunner_context.settings
        if settings.notifications:
            configs = []
            if settings.notifications.emails:
                configs.append(
                    NotificationTargetModel(
                        type=NotificationsType.EMAIL_RECIPIENTS.value, value=settings.notifications.emails
                    )
                )
            if settings.notifications.slack_channels:
                configs.append(
                    NotificationTargetModel(
                        type=NotificationsType.SLACK_CHANNELS.value, value=settings.notifications.slack_channels
                    )
                )
            if settings.notifications.ms_teams_channels:
                configs.append(
                    NotificationTargetModel(
                        type=NotificationsType.MS_TEAMS_CHANNELS.value, value=settings.notifications.ms_teams_channels
                    )
                )
            return configs
        else:
            return None

    def start_test_run(self) -> None:
        self.authorize()
        settings = zebrunner_context.settings
        test_run = TestRun(settings.run.display_name, settings.run.environment, settings.run.build)
        zebrunner_context.test_run = test_run
        milestone = (
            MilestoneModel(id=settings.milestone.id, name=settings.milestone.name) if settings.milestone else None
        )

        start_run_model = StartTestRunModel(
            name=test_run.name,
            framework="pytest",
            config=TestRunConfigModel(environment=test_run.environment, build=test_run.build),
            milestone=milestone,
            ci_context=resolve_ci_context(),
            notification_targets=self.get_notification_configurations(),
        )

        if settings.run.context:
            zebrunner_run_context = self.api.get_rerun_tests(settings.run.context)
            start_run_model.uuid = zebrunner_run_context.test_run_uuid
            if not zebrunner_run_context.run_allowed:
                pytest.exit(f"Run not allowed by zebrunner! Reason: {zebrunner_run_context.reason}")
            if zebrunner_run_context.run_only_specific_tests and not zebrunner_run_context.tests_to_run:
                pytest.exit("Aborted. No tests to run!!")
        test_run.zebrunner_id = self.api.start_test_run(settings.project_key, start_run_model)

    def start_test(self, report: TestReport) -> None:
        self.authorize()
        test = Test(
            name=report.nodeid.split("::")[1],
            file=report.nodeid.split("::")[0],
            maintainers=report.maintainers,
            labels=report.labels,
        )
        zebrunner_context.test = test

        if zebrunner_context.test_run_is_active:
            test.zebrunner_id = self.api.start_test(
                zebrunner_context.test_run_id,
                StartTestModel(
                    name=test.name,
                    class_name=test.file,
                    method_name=test.name,
                    maintainer=",".join(test.maintainers),
                    labels=[{"key": label[0], "value": label[1]} for label in test.labels],
                    correlation_data=CorrelationDataModel(name=test.name).json(),
                ),
            )

            if report.artifact_references:
                references = [ArtifactReferenceModel(name=x[0], value=x[1]) for x in report.artifact_references]
                self.api.send_artifact_references(references, zebrunner_context.test_run_id, zebrunner_context.test_id)
            if report.artifacts:
                for artifact in report.artifacts:
                    self.api.send_artifact(artifact, zebrunner_context.test_run_id, zebrunner_context.test_id)

    def finish_test(self, report: TestReport) -> None:
        if zebrunner_context.test_is_active:
            self.authorize()
            is_skip = report.when == "setup" and report.outcome == "skipped"
            is_xfail = hasattr(report, "wasxfail")

            reason = None
            if report.passed:
                status = TestStatus.PASSED
            elif is_skip or is_xfail:
                status = TestStatus.SKIPPED
            else:
                status = TestStatus.FAILED

            if is_xfail:
                reason = report.wasxfail
            elif is_skip:
                reason = report.longrepr[-1] if isinstance(report.longrepr, tuple) else str(report.longrepr)
            else:
                # Following this changelog check if it's string or ReprExceptionInfo
                # https://docs.pytest.org/en/6.2.x/changelog.html?highlight=reprexceptioninfo#pytest-6-0-0rc1-2020-07-08
                reason = str(report.longrepr)
                if isinstance(report.longrepr, ReprExceptionInfo) or isinstance(report.longrepr, ExceptionChainRepr):
                    reason = report.longrepr.reprcrash.message + "\n\n" + reason

            self.api.finish_test(
                zebrunner_context.test_run_id,
                zebrunner_context.test_id,
                FinishTestModel(
                    result=status.value,
                    reason=reason,
                ),
            )
            zebrunner_context.test = None

    def finish_test_run(self) -> None:
        self.authorize()
        if zebrunner_context.test_run_is_active:
            self.api.finish_test_run(zebrunner_context.test_run_id)

            handlers = list(filter(lambda x: isinstance(x, ZebrunnerHandler), logging.root.handlers))
            if len(handlers) > 0:
                zebrunner_handler: ZebrunnerHandler = handlers[0]  # type: ignore
                zebrunner_handler.push_logs()

        self.api.close()

    def start_test_session(
        self, session_id: str, capabilities: dict, desired_capabilities: dict, test_ids: List[int]
    ) -> Optional[str]:
        self.authorize()
        if zebrunner_context.test_run_is_active:
            zebrunner_session_id = self.api.start_test_session(
                zebrunner_context.test_run_id,
                StartTestSessionModel(
                    session_id=session_id,
                    desired_capabilities=desired_capabilities,
                    capabilities=capabilities,
                    test_ids=test_ids,
                ),
            )
            return zebrunner_session_id
        return None

    def finish_test_session(self, zebrunner_session_id: str, related_tests: List[str]) -> None:
        self.authorize()
        if zebrunner_context.test_run_is_active:
            self.api.finish_test_session(
                zebrunner_context.test_run_id,
                zebrunner_session_id,
                FinishTestSessionModel(test_ids=related_tests),
            )

    def filter_test_items(self, items: List[Item]) -> List[Item]:
        run_context = zebrunner_context.settings.run.context
        if run_context is not None:
            self.authorize()
            context_data = self.api.get_rerun_tests(run_context)
            rerun_names = {
                x.correlation_data.name for x in context_data.tests_to_run if x.correlation_data is not None
            }
            return list(filter(lambda x: x.name in rerun_names, items)) if rerun_names else items
        else:
            return items



from pytest_zebrunner.attachments import (
    attach_test_artifact,
    attach_test_artifact_reference,
    attach_test_label,
    attach_test_run_artifact,
    attach_test_run_artifact_reference,
    attach_test_run_label,
    attach_test_screenshot,
)

__version__ = "1.4.1"
__all__ = [
    "attach_test_artifact_reference",
    "attach_test_run_artifact_reference",
    "attach_test_label",
    "attach_test_run_label",
    "attach_test_artifact",
    "attach_test_run_artifact",
    "attach_test_screenshot",
]



import logging

import pytest
from _pytest.main import Session
from _pytest.nodes import Item
from _pytest.reports import TestReport
from _pytest.runner import CallInfo

from pytest_zebrunner.context import TestRun, zebrunner_context
from pytest_zebrunner.reporting_service import ReportingService
from pytest_zebrunner.selenium_integration import SeleniumSession, inject_driver

logger = logging.getLogger(__name__)


class PytestHooks:
    def __init__(self) -> None:
        self.service = ReportingService()
        self.session_manager = SeleniumSession(self.service)
        self.is_worker = True
        self.is_controller = False

    @pytest.hookimpl
    def pytest_sessionstart(self, session: Session) -> None:
        self.is_worker = hasattr(session.config, "workerinput")
        self.is_controller = getattr(session.config.option, "dist", "no") != "no"
        if self.is_worker:
            test_run = TestRun(
                zebrunner_context.settings.run.display_name,
                zebrunner_context.settings.run.environment,
                zebrunner_context.settings.run.build,
            )
            test_run.zebrunner_id = session.config.workerinput["test_run_id"]
            zebrunner_context.test_run = test_run
        else:
            self.service.start_test_run()

        inject_driver(self.session_manager)

    @pytest.hookimpl
    def pytest_collection_finish(self, session: Session) -> None:
        session.items = self.service.filter_test_items(session.items)

    @pytest.hookimpl
    def pytest_sessionfinish(self, session: Session, exitstatus: int) -> None:
        if not self.is_worker:
            self.service.finish_test_run()
            self.session_manager.finish_all_sessions()

    @pytest.mark.hookwrapper
    def pytest_runtest_makereport(self, item: Item, call: CallInfo) -> TestReport:
        outcome = yield
        report: TestReport = outcome.get_result()

        report.maintainers = [str(mark.args[0]) for mark in item.iter_markers("maintainer")]
        report.labels = [(str(mark.args[0]), str(mark.args[1])) for mark in item.iter_markers("label")]
        report.artifact_references = [
            (str(mark.args[0]), str(mark.args[1])) for mark in item.iter_markers("artifact_reference")
        ]
        report.artifacts = [mark.args[0] for mark in item.iter_markers("artifact")]

    @pytest.hookimpl
    def pytest_runtest_logreport(self, report: TestReport) -> None:
        if self.is_worker or not self.is_controller:
            is_setup_rerun = hasattr(report, "rerun") and report.rerun > 0
            is_call_rerun = report.outcome == "rerun"
            if report.when == "setup" and not is_setup_rerun:
                self.service.start_test(report)
                if report.outcome == "skipped":
                    self.service.finish_test(report)
            elif report.when == "call" and not is_call_rerun:
                self.service.finish_test(report)


class XdistHooks:
    def pytest_configure_node(self, node):  # type: ignore
        node.workerinput["test_run_id"] = zebrunner_context.test_run.zebrunner_id



from datetime import datetime
from typing import List, Optional, Tuple

from pydantic.error_wrappers import ValidationError

from pytest_zebrunner.settings import load_settings


class TestRun:
    def __init__(self, name: str = None, environment: str = None, build: str = None) -> None:
        self.zebrunner_id: Optional[int] = None
        self.name = name or f"Unnamed {datetime.utcnow()}"
        self.environment = environment
        self.build = build


class Test:
    def __init__(self, name: str, file: str, maintainers: List[str], labels: List[Tuple[str, str]]) -> None:
        self.zebrunner_id: Optional[int] = None
        self.name = name
        self.file = file
        self.maintainers = maintainers
        self.labels = labels


class ZebrunnerContext:
    def __init__(self) -> None:
        self.test_run: Optional[TestRun] = None
        self.test: Optional[Test] = None
        try:
            self.settings = load_settings()
        except ValidationError:
            self.settings = None  # type: ignore

    @property
    def test_is_active(self) -> bool:
        return self.test_run_is_active and self.test is not None and self.test.zebrunner_id is not None

    @property
    def test_run_is_active(self) -> bool:
        return self.test_run is not None and self.test_run.zebrunner_id is not None

    @property
    def test_id(self) -> int:
        if self.test is not None and self.test.zebrunner_id is not None:
            return self.test.zebrunner_id
        else:
            raise ValueError("Test id not found.")

    @property
    def test_run_id(self) -> int:
        if self.test_run is not None and self.test_run.zebrunner_id is not None:
            return self.test_run.zebrunner_id
        else:
            raise ValueError("Test-run id not found")


zebrunner_context = ZebrunnerContext()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pytest_zebrunner', 'pytest_zebrunner.api']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.3.1,<6.0.0',
 'httpx>=0.15.0,<0.16.0',
 'pydantic>=1.0',
 'pytest>=4.5.0',
 'python-dotenv>=0.10']

entry_points = \
{'pytest11': ['pytest-zebrunner = pytest_zebrunner.plugin']}

setup_kwargs = {
    'name': 'pytest-zebrunner',
    'version': '1.4.1',
    'description': 'Pytest connector for Zebrunner reporting',
    'long_description': '# Zebrunner PyTest agent\n\n\nThe official Zebrunner PyTest agent provides the reporting functionality. It can automatically track Selenium sessions\nand send the info about session details to Zebrunner backend. It can be easily integrated into a project by just installing the library\nand adding the configuration file.\n\nIncluding reporting into your project is easy - just install the agent and provide minimal valid configuration for reporting.\n\n\n## Installation\n\n    pip install pytest-zebrunner\n\n\n## Configuration\nAfter the installation, reporting is disabled by default. It won\'t send any data to the Zebrunner service without a valid configuration.\n\nIt is currently possible to provide the configuration via:\n1. Environment variables\n2. YAML file\n\n`pyproject.toml`, `command arguments` are in plans for future releases.\n\n\n\n<!-- groups:start -->\n### Environment variables\nThe following configuration parameters are recognized by the agent:\n\n- `REPORTING_ENABLED` - enables or disables reporting. The default value is `true`.\n- `REPORTING_SERVER_HOSTNAME` - mandatory if reporting is enabled. It is Zebrunner server hostname. It can be obtained in Zebrunner on the \'Account & profile\' page under the \'Service URL\' section;\n- `REPORTING_SERVER_ACCESS_TOKEN` - mandatory if reporting is enabled. Access token must be used to perform API calls. It can be obtained in Zebrunner on the \'Account & profile\' page under the \'Token\' section;\n- `REPORTING_PROJECT_KEY` - optional value. It is the project that the test run belongs to. The default value is `DEF`. You can manage projects in Zebrunner in the appropriate section;\n- `REPORTING_RUN_DISPLAY_NAME` - optional value. It is the display name of the test run. The default value is `Default Suite`;\n- `REPORTING_RUN_BUILD` - optional value. It is the build number that is associated with the test run. It can depict either the test build number or the application build number;\n- `REPORTING_RUN_ENVIRONMENT` - optional value. It is the environment where the tests will run;\n- `REPORTING_SEND_LOGS` - Sends test logs to Zebrunner. Default: `true`;\n\nAgent also recognizes `.env` file in the resources root folder.\n<!-- groups:end -->\n\n<!-- groups:start -->\n### Yaml file\nAgent recognizes agent.yaml or agent.yml file in the resources root folder. It is currently not possible to configure an alternative file location.\n\nBelow is a sample configuration file:\n```yaml\nreporting:\n  enabled: true\n  project-key: DEF\n  send-logs: true\n  server:\n    hostname: localhost:8080\n    access-token: <token>\n  run:\n    display-name: Nightly Regression Suite\n    build: 1.12.1.96-SNAPSHOT\n    environment: TEST-1\n```\n<!-- groups:end -->\n\n- `reporting.enabled` - enables or disables reporting. The default value is `true`;\n- `reporting.server.hostname` - mandatory if reporting is enabled. Zebrunner server hostname. Can be obtained in Zebrunner on the \'Account & profile\' page under the \'Service URL\' section;\n- `reporting.server.access-token` - mandatory if reporting is enabled. Access token must be used to perform API calls. Can be obtained in Zebrunner on the \'Account & profile\' page under the \'Token\' section;\n- `reporting.project-key` - optional value. The project that the test run belongs to. The default value is `DEF`. You can manage projects in Zebrunner in the appropriate section;\n- `reporting.send-logs` - Sends test logs to Zebrunner. Default: `true`\n- `reporting.run.display-name` - optional value. The display name of the test run. The default value is Default Suite;\n- `reporting.run.build` - optional value. The build number that is associated with the test run. It can depict either the test build number or the application build number;\n- `reporting.run.environment` - optional value. The environment in which the tests will run.\n\nIf the required configurations are not provided, there is a warning displayed in logs with the problem description and the names of options\nwhich need to be specified. Parameter names are case insensitive and can be written in upper and lower registers.\n\n### Advanced configuration example\nSometimes there is need to change configuration from run to run. This can be done with changing environment variables\nbefore each run.\n```python\nimport os\nfrom datetime import datetime\nimport random\n\nimport dotenv\nimport pytest\n\n\n# Generate your run name here\ndef get_run_name():\n    return f"Regression [{datetime.now()}] [Other helpull stuff]"\n\n# Generate your build number here\ndef get_build_number():\n    return f"{random.randint(1, 10)}.{random.randint(1, 10)}.{random.randint(1, 10)}"\n\ndef load_access_token():\n    return dotenv.dotenv_values("secrets/secrets.txt")["REPORTING_SERVER_ACCESS_TOKEN"]\n\n\ndef run_tests():\n    os.environ["REPORTING_RUN_DISPLAY_NAME"] = get_run_name()\n    os.environ["REPORTING_RUN_BUILD"] = get_build_number()\n\n    # If you store your secrets seperately you can load it here\n    dotenv.load_dotenv("secrets/secrets.env")\n    # or you can set directly\n    os.environ["REPORTING_SERVER_ACCESS_TOKEN"] = load_access_token()\n\n    # Here you can pass arguments to pytest\n    pytest.main(["-n", "-2"])\n\n\nif __name__ == "__main__":\n    run_tests()\n```\n\n## Collecting logs\nFor sending logs to zebrunner you need to add ZebrunnerHandler to yours logger.\nExample:\n```python\nimport logging\n\nfrom pytest_zebrunner.zebrunner_logging import ZebrunnerHandler\n\nlogger = logging.getLogger(__name__)\nlogger.addHandler(ZebrunnerHandler())\n```\n\nTo send all logs to zebrunner you can add `ZebrunnerHandler` to root logger.\n```python\nimport logging\n\nfrom pytest_zebrunner.zebrunner_logging import ZebrunnerHandler\n\nlogging.root.addHandler(ZebrunnerHandler())\n```\n\n\n## Additional functionality\n\n**IMPORTANT**: All attachments to tests can only be done while some test is running.\nAll attachments to a test run can only be done while a pytest test session is active.\n---------------------------\n\n### Collecting captured screenshot\nSometimes it may be useful to have the ability to track captured screenshots in scope of Zebrunner Reporting. The agent comes\nwith the API allowing you to send your screenshots to Zebrunner, so that they could be attached to the test.\n\n```python\nfrom pytest_zebrunner import attach_test_screenshot\n\n\ndef test_something():\n    ...\n    driver.save_screenshot("path_to_screenshot.png) # Capture screenshot with selenium driver\n    attach_test_screenshot("path_to_screenshot.png")\n    ...\n```\n\n### Collecting additional artifacts\nIn case your tests or an entire test run produce some artifacts, it may be useful to track them in Zebrunner.\nThe agent comes with a few convenient methods for uploading artifacts in Zebrunner and linking them to the currently running test or test run.\nArtifacts and artifact references can be attached using functions from `attachments` module. Together with an artifact\nor artifact reference, you must provide the display name. For the file, this name must contain the file extension that\nreflects the actual content of the file. If the file extension does not match the file content, this file will not be\nsaved in Zebrunner. Artifact reference can have an arbitrary name.\n\n#### Attaching artifact to test\n```python\nfrom pytest_zebrunner import attach_test_artifact\n\n\ndef test_something():\n    ...\n    attach_test_artifact("path_to_artifact")\n    ...\n```\nor with mark:\n```python\n@pytest.mark.artifact("path_to_artifact")\ndef test_something():\n    ...\n```\n\n### Attaching artifact reference to test\n```python\nfrom pytest_zebrunner import attach_test_artifact_reference\n\n\ndef test_something():\n    ...\n    attach_test_artifact_reference("name", "reference")\n    ...\n```\nor with mark:\n```python\n@pytest.mark.artifact_reference("name", "reference")\ndef test_something():\n    ...\n```\n\n### Attaching artifact to test run\n```python\nfrom pytest_zebrunner import attach_test_run_artifact\n\n\nattach_test_run_artifact("path_to_artifact")\n```\n\n### Attaching artifact reference to test run\n```python\nfrom pytest_zebrunner import attach_test_run_artifact_reference\n\n\nattach_test_run_artifact_reference("name", "reference")\n```\n\nArtifact uploading process is performed in the foreground now, so it will block the execution thread while sending.\nThe background uploading will be available in upcoming releases.\n\n\n### Attaching test labels\nIn some cases, it may be useful to attach some meta information related to a test. The agent comes with a concept of a label.\nLabel is a key-value pair associated with a test. The key and value are represented by a `str`. Labels can be attached to\ntests and test runs.\n\n```python\n@pytest.mark.label("name", "value")\ndef test_something():\n    ...\n```\nor\n```python\nfrom pytest_zebrunner import attach_test_label\n\n\ndef test_something():\n    ...\n    attach_test_label("name", "value")\n    ...\n```\n**Note:** These two methods can be combined.\n\nFor test run:\n```python\nfrom pytest_zebrunner import attach_test_run_label\n\nattach_test_run_label("name", "value")\n```\n\n\n### Tracking of test maintainer\nYou may want to add transparency to the process of automation maintenance by having an engineer responsible for\nevolution of specific tests or test classes. Zebrunner comes with a concept of a maintainer - a person that can be\nassigned to maintain tests. In order to keep track of those, the agent comes with the `@pytest.mark.maintainer` annotation.\n\nSee a sample test bellow:\n\n```python\n@pytest.mark.maintainer("username_of_maintainer")\ndef test_something():\n    ...\n```\n\n### Tracking web driver sessions\nThe Zebrunner test agent has a great ability to track tests along with remote driver sessions. You don\'t have to do anything.\nThe agent automatically injects the tracking functionality to the Selenium driver if the Selenium library is installed. The agent sends\ndriver capabilities to Zebrunner when the driver starts and the finish time when the driver stops.\n\nZebrunner can automatically collect your test artifacts like videos or selenium session logs if you are using\ntest running providers like ZebrunnerHub, BrowserStack, Lambdatest, etc. To enable this you need to enable integration in your project in Zebrunner\nand also specify the `provider` capability.\n\n#### Zebrunner\nExample:\n```python\ndef test_something():\n    hub_url = \'https://username:password@engine.zebrunner.com/wd/hub\'\n    capabilities = {\n        \'browserName\': \'firefox\',\n        \'enableVideo\': True,\n        \'enableLog\': True,\n        \'enableVNC\': True,\n        \'provider\': \'zebrunner\',\n        ...\n    }\n    driver = Remote(command_executor=hub_url, desired_capabilities=capabilities)\n    ...\n```\n\n### BrowserStack\nBrowserStack saves video and logs by default so you need just to specify capability.\nExample:\n```python\ndef test_something():\n    hub_url = \'https://username:password@hub-cloud.browserstack.com/wd/hub\'\n    capabilities = {\n        \'browser\': \'firefox\',\n        \'provider\': \'BROWSERSTACK\',\n        ...\n    }\n    driver = Remote(command_executor=hub_url, desired_capabilities=capabilities)\n    ...\n```\n',
    'author': 'Anatoliy Platonov',
    'author_email': 'p4m.dev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://zebrunner.com/documentation/agents/pytest',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import json
import logging
import time
from datetime import datetime, timedelta, timezone
from pathlib import Path
from pprint import pformat
from typing import List, Optional, Union

import httpx
from httpx import Client, Request, Response

from pytest_zebrunner.api.models import (
    ArtifactReferenceModel,
    FinishTestModel,
    FinishTestSessionModel,
    LabelModel,
    LogRecordModel,
    RerunDataModel,
    StartTestModel,
    StartTestRunModel,
    StartTestSessionModel,
)
from pytest_zebrunner.utils import Singleton

logger = logging.getLogger(__name__)


def log_response(response: Response, log_level: int = logging.DEBUG) -> None:
    """
    Logger customized configuration.

    Args:
         response (Response): Http response from server to client
         log_level (int): Logging messages which are less severe than level will be ignored.

    Attributes:
        request (Request): Request instance associated to the current response.
    """
    request = response.request
    request.read()

    logger.log(
        log_level,
        f"Request {request.method} {request.url}\n"
        f"Headers: \n{pformat(dict(request.headers))}\n\n"
        f"Content: \n{request.content}\n\n"
        f"Response Code: {response.status_code}\n"
        f" Content: \n{pformat(response.json())}",
    )


class ZebrunnerAPI(metaclass=Singleton):
    """
    A class used to represent ZebrunnerAPI using Singleton Pattern which guarantees that will exist only one instance
    of this class.

    Attributes:
        authenticated (bool): True when a valid access token is given.

    """

    authenticated = False

    def __init__(self, service_url: str = None, access_token: str = None):
        """
        Args:
            service_url (str): Url to access Zebrunner API. None by default.
            access_token (str): Access token to access Zebrunner API. None by default.
        """
        if service_url and access_token:
            self.service_url = service_url.rstrip("/")
            self.access_token = access_token
            self._client = Client()
            self._auth_token = None
            self.authenticated = False

    def _sign_request(self, request: Request) -> Request:
        """
        Returns a request with the _auth_token set to the authorization request header.

        Args:
            request (Request):

        Returns:
            request (Request): Request whose authorization request header has been set with _auth_token.

        """
        request.headers["Authorization"] = f"Bearer {self._auth_token}"
        return request

    def auth(self) -> None:
        """
        Validates the user access token with http post method and if it is correct, authenticates the user.

        Attributes:
            url (str): Url to validates the user access token.

        """
        if not self.access_token or not self.service_url:
            return

        url = self.service_url + "/api/iam/v1/auth/refresh"
        try:
            response = self._client.post(url, json={"refreshToken": self.access_token})
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return

        self._auth_token = response.json()["authToken"]
        self._client.auth = self._sign_request
        self.authenticated = True

    def start_test_run(self, project_key: str, body: StartTestRunModel) -> Optional[int]:
        """
        Execute an http post with the given project_key and body, which contains StartTestRunModel.
        If everything is OK, returns response id value for test run. Otherwise, logs errors and returns None.

        Args:
            project_key (str):
            body (StartTestRunModel): Entity with TestRun properties.

        Attributes:
            url (str): Url to access api reporting test runs.

        Returns:
            (int, optional): Returns http response 'id' value if http post was OK. Otherwise, returns None.
        """
        url = self.service_url + "/api/reporting/v1/test-runs"

        try:
            response = self._client.post(
                url, params={"projectKey": project_key}, json=body.dict(exclude_none=True, by_alias=True)
            )
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return None

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return None

        return response.json()["id"]

    def start_test(self, test_run_id: int, body: StartTestModel) -> Optional[int]:
        """
        Execute an http post with the given test_run_id and body, which contains StartTestModel.
        If everything is OK, returns response id value for test. Otherwise, logs errors and returns None.

        Args:
            test_run_id (int): Number that identifies test_run.
            body (StartTestModel): Entity with Test properties.

        Returns:
            (int, optional): Returns http response 'id' value if http post was OK. Otherwise, returns None.
        """
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/tests"

        try:
            response = self._client.post(url, json=body.dict(exclude_none=True, by_alias=True))
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return None

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return None

        return response.json()["id"]

    def finish_test(self, test_run_id: int, test_id: int, body: FinishTestModel) -> None:
        """
        Execute an http put with the given test_run_id, test_id, and body, which contains FinishTestModel.
        If everything is OK, finish the Test. Otherwise, logs errors.

        Args:
            test_run_id (int): Number that identifies test_run.
            test_id (int): Number that identifies test.
            body (FinishTestModel): Entity with FinishTest properties.

        """
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}"

        try:
            response = self._client.put(url, json=body.dict(exclude_none=True, by_alias=True))
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return

        if response.status_code != 200:
            log_response(response, logging.ERROR)

    def finish_test_run(self, test_run_id: int) -> None:
        """
        Execute an http put with the given test_run_id.
        If everything is OK, updates endedAt value finishing test_run. Otherwise, logs errors.

        Args:
            test_run_id (int): Number that identifies test_run.
        """
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}"
        try:
            response = self._client.put(
                url,
                json={"endedAt": (datetime.utcnow().replace(tzinfo=timezone.utc) - timedelta(seconds=1)).isoformat()},
            )
        except httpx.RequestError as e:
            logger.warning("Error while sending request to zebrunner.", exc_info=e)
            return

        if response.status_code != 200:
            log_response(response, logging.ERROR)
            return

    def send_logs(self, test_run_id: int, logs: List[LogRecordModel]) -> None:
        """
        Convert a list of LogRecordModel to dictionary format(json) and send them to logs endpoint
        associated with the appropriate test_run_id, for reporting.

        Args:
            test_run_id (int): Number that identifies test_run.
            logs (List[LogRecordModel]): List of LogRecordModel to send for reporting.
        """
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/logs"

        body = [x.dict(exclude_none=True, by_alias=True) for x in logs]
        self._client.post(url, json=body)

    def send_screenshot(self, test_run_id: int, test_id: int, image_path: Union[str, Path]) -> None:
        """
        Open an image file by its path, read the binary content and send it to screenshots endpoint
        associated with the appropriate test_id, for reporting.

        Args:
            test_run_id (int): Number that identifies test_run.
            test_id (int): Number that identifies test.
            image_path (Union[str,Path)]: Path to identify image location in directory structure.

        Raises:
            FileNotFoundError: If screenshot file is not reachable.

        """
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/screenshots"
        with open(image_path, "rb") as image:
            self._client.post(
                url,
                content=image.read(),
                headers={"Content-Type": "image/png", "x-zbr-screenshot-captured-at": round(time.time() * 1000)},
            )

    def send_artifact(self, filename: Union[str, Path], test_run_id: int, test_id: Optional[int] = None) -> None:
        """
        Open a file by its path, read the binary content and sent it to artifacts endpoint
        associated with the appropriate test_id if one is given.
        Otherwise the artifacts endpoint is the one associated with only the appropriate test_run_id.

        Args:
            filename (Union[str, Path]): Path to identify artifact location in directory structure.
            test_run_id (int): Number that identifies test_run.
            test_id: (int, optional): Number that identifies test.

        Raises:
            FileNotFoundError: If artifact file is not reachable.

        """
        if test_id:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/artifacts"
        else:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/artifacts"
        with open(filename, "rb") as file:
            self._client.post(url, files={"file": file})

    def send_artifact_references(
        self, references: List[ArtifactReferenceModel], test_run_id: int, test_id: Optional[int] = None
    ) -> None:
        """
        Convert a list of ArtifactsReferenceModel to dictionary format(json) and send them to
        test artifact-references endpoint associated with the appropriate test_id if one is given.
        Otherwise the artifact-references endpoint is the one associated with only the appropriate test_run_id.

        Args:
            references (List[ArtifactReferenceModel]): List of artifacts references to send for reporting.
            test_run_id (int): Number that identifies test_run.
            test_id: (int, optional): Number that identifies test.
        """
        if test_id:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/artifact-references"
        else:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/artifact-references/"
        json_items = [item.dict(exclude_none=True, by_alias=True) for item in references]
        self._client.put(url, json={"items": json_items})

    def send_labels(self, labels: List[LabelModel], test_run_id: int, test_id: Optional[int] = None) -> None:
        """
        Convert a list of LabelModel to dictionary format(json) and send them to
        labels endpoint associated with the appropiate test_id if one is given.
        Otherwise, the labels endpoint is the one associated with only the appropriate test_run_id.

        Args:
            labels (List[LabelModel]): List of labels to send for reporting.
            test_run_id (int): Number that identifies test_run.
            test_id: (optional int): Number that identifies test.

        """
        if test_id:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/tests/{test_id}/labels"
        else:
            url = f"{self.service_url}/api/reporting/v1/test-runs/{test_run_id}/labels"
        labels_json = [label.dict(exclude_none=True, by_alias=True) for label in labels]
        self._client.put(url, json={"items": labels_json})

    def start_test_session(self, test_run_id: int, body: StartTestSessionModel) -> Optional[str]:
        """
        Execute an http post with the given test_run_id and body, which contains StartTestSessionModel.
        If everything is OK, returns response id value for test. Otherwise, logs errors and returns None.

        Args:
            test_run_id (int): Number that identifies test_run.
            body (StartTestSessionModel): Entity with TestSession properties.

        Returns:
            (string, optional): Returns http response 'id' value if http post was OK. Otherwise, returns None.
        """
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/test-sessions"
        response = self._client.post(url, json=body.dict(exclude_none=True, by_alias=True))
        if not response.status_code == 200:
            log_response(response, logging.ERROR)
            return None

        return response.json().get("id")

    def finish_test_session(self, test_run_id: int, zebrunner_id: str, body: FinishTestSessionModel) -> None:
        """
        Execute an http put with the given test_run_id, zebrunner_id and body, which contains FinishTestSessionModel.
        If everything is OK, finish the test_session.

        Args:
            test_run_id (int): Number that identifies test_run.
            zebrunner_id (str): String that identifies session.
        """
        url = self.service_url + f"/api/reporting/v1/test-runs/{test_run_id}/test-sessions/{zebrunner_id}"
        self._client.put(url, json=body.dict(exclude_none=True, by_alias=True))

    def get_rerun_tests(self, run_context: str) -> RerunDataModel:
        """"""
        url = self.service_url + "/api/reporting/v1/run-context-exchanges"
        run_context_dict = json.loads(run_context)
        response = self._client.post(url, json=run_context_dict)
        response_data = response.json()
        for test in response_data["tests"]:
            correlation_data = test["correlationData"]
            if correlation_data is not None:
                test["correlationData"] = json.loads(correlation_data)
        return RerunDataModel(**response_data)

    def close(self) -> None:
        """
        Close the connection pool without block-usage.
        """
        self._client.close()



import logging
import warnings

from _pytest.config import Config
from pydantic import ValidationError

from pytest_zebrunner.context import zebrunner_context
from pytest_zebrunner.hooks import PytestHooks, XdistHooks
from pytest_zebrunner.settings import load_settings

logger = logging.getLogger(__name__)


def pytest_configure(config: Config) -> None:

    try:
        settings = load_settings()
        zebrunner_context.settings = settings
    except ValidationError as exc:
        field_errors = "\n".join([f"\033[93m {e['loc'][0]}\033[0m - {e['msg']}" for e in exc.errors()])
        warnings.warn(
            UserWarning(
                "\033[1;31m Zebrunner plugin not configured properly because missing required config options.\n"
                "Add it to environment variables or .env file.\n" + field_errors + "\n" * 3
            )
        )
        return

    if settings.enabled:
        hooks = PytestHooks()

        config.pluginmanager.register(hooks)
        if config.pluginmanager.get_plugin("xdist") is not None:
            config.pluginmanager.register(XdistHooks())

        config.addinivalue_line("markers", "maintainer(name): Email or nickname of test maintainer")
        config.addinivalue_line("markers", "label(name, value): Test label")
        config.addinivalue_line("markers", "artifact(path): Attach file to the test")
        config.addinivalue_line("markers", "artifact_reference(name, value): Attach reference to the test")



import copy
import logging
from typing import Any, Dict

from pytest_zebrunner.context import zebrunner_context

logger = logging.getLogger(__name__)


class SeleniumSession:
    def __init__(self, reporting_service) -> None:  # type: ignore
        self._active_sessions: Dict[str, Any] = {}
        self.reporting_service = reporting_service

    def start_session(self, session_id: str, capabilities: dict, desired_capabilities: dict) -> None:
        self._active_sessions[session_id] = {"related_tests": []}
        if zebrunner_context.test_is_active:
            self._active_sessions[session_id]["related_tests"].append(zebrunner_context.test_id)

        zebrunner_session_id = self.reporting_service.start_test_session(
            session_id, capabilities, desired_capabilities, self._active_sessions[session_id]["related_tests"]
        )
        if zebrunner_session_id:
            self._active_sessions[session_id]["zebrunner_session_id"] = zebrunner_session_id

    def finish_session(self, session_id: str) -> None:
        self.reporting_service.finish_test_session(
            self._active_sessions[session_id]["zebrunner_session_id"],
            self._active_sessions[session_id]["related_tests"],
        )
        del self._active_sessions[session_id]

    def finish_all_sessions(self) -> None:
        for session_id in list(self._active_sessions):
            self.finish_session(session_id)

    def add_test(self, test_id: int) -> None:
        for session_id in self._active_sessions:
            if self._active_sessions[session_id].get("related_tests") is not None:
                self._active_sessions[session_id]["related_tests"].append(test_id)
            else:
                self._active_sessions[session_id]["related_tests"] = [test_id]


def inject_driver(session_manager: SeleniumSession) -> None:
    try:
        from selenium.webdriver.remote.webdriver import WebDriver

        base_init = WebDriver.__init__
        base_quit = WebDriver.quit

        def init(  # type: ignore
            session: WebDriver,
            command_executor: str = "http://127.0.0.1:4444/wd/hub",
            desired_capabilities: dict = None,
            browser_profile=None,
            proxy=None,
            keep_alive: bool = False,
            file_detector=None,
            options=None,
        ) -> None:  # type: ignore
            # Override capabilities and command_executor with new ones provided by zebrunner.
            caps = copy.deepcopy(desired_capabilities)
            if zebrunner_context.settings.zebrunner:
                zeb_settings = zebrunner_context.settings.zebrunner
                if zeb_settings.hub_url:
                    command_executor = zeb_settings.hub_url
                if zeb_settings.desired_capabilities and caps:
                    caps.update(zeb_settings.desired_capabilities)

            base_init(
                session,
                command_executor,
                caps,
                browser_profile,
                proxy,
                keep_alive,
                file_detector,
                options,
            )
            session_manager.start_session(session.session_id, session.capabilities, desired_capabilities or {})
            if zebrunner_context.test_is_active:
                session_manager.add_test(zebrunner_context.test_id)

        def quit(session) -> None:  # type: ignore
            session_manager.finish_session(session.session_id)
            base_quit(session)

        WebDriver.__init__ = init
        WebDriver.quit = quit

    except ImportError:
        logger.warning("Selenium library is not installed.")



import os
from enum import Enum
from typing import Dict, List, Optional, Type

from pytest_zebrunner.api.models import CiContextModel


class BaseContextLoader:
    """
    A class use as base to load environment variables.
    """

    @staticmethod
    def load_context_variables(prefixes: List[str]) -> Dict[str, str]:
        """
        Returns a dictionary with environment variables that start with some prefix.

        Args:
            prefixes (List[str]): List of prefixes to find environment variables.

        Returns:
            (Dict[str,str]): Environment variables in key-value format.
        """
        env_variable_names = list(filter(lambda name: any([name.startswith(x) for x in prefixes]), os.environ))
        return {key: os.environ[key] for key in env_variable_names}


class CiType(Enum):
    """
    A class that inherit from Enum, used to represent CI constants
    """

    JENKINS = "JENKINS"
    TEAM_CITY = "TEAM_CITY"
    CIRCLE_CI = "CIRCLE_CI"
    TRAVIS_CI = "TRAVIS_CI"
    BAMBOO = "BAMBOO"


class JenkinsContextLoader(BaseContextLoader):
    """
    A class that inherit from BaseContextLoader,used to represent JenkinsContext with its own environment variables.

    Attributes:
        CI_ENV_VARIABLE (str): 'JENKINS_URL'.
        CI_TYPE (CiType):
        ENV_VARIABLE_PREFIXES (List[str]): List of prefixes to find and load Jenkins environment variables.
    """

    CI_ENV_VARIABLE = "JENKINS_URL"
    CI_TYPE = CiType.JENKINS
    ENV_VARIABLE_PREFIXES = [
        "CVS_",
        "SVN_",
        "GIT_",
        "NODE_",
        "EXECUTOR_NUMBER",
        "JENKINS_",
        "JOB_",
        "BUILD_",
        "ROOT_BUILD_",
        "RUN_",
        "WORKSPACE",
    ]

    @classmethod
    def resolve(cls) -> Optional[Dict[str, str]]:
        """
        Returns a dictionary with Jenkins environment variables if 'cls.JENKINS_URL' exists
        in operating system environment. Otherwise returns None.

        Args:
            cls (JenkinsContextLoader):

        Returns:
            Union(Dict[str, str], optional): Jenkins environment variables in key-value format or None if 'JENKINS_URL'
            does not exists in os.environment.
        """
        if cls.CI_ENV_VARIABLE in os.environ:
            return cls.load_context_variables(cls.ENV_VARIABLE_PREFIXES)
        else:
            return None


class TeamCityCiContextResolver(BaseContextLoader):
    """
    A class that inherit from BaseContextLoader,used to represent TeamCityCIContext with its own environment variables.

    Attributes:
        CI_ENV_VARIABLE (str): 'TEAMCITY_VERSION'
        CI_TYPE (CiType):
        ENV_VARIABLE_PREFIXES (List[str]): List of prefixes to find and load Team City environment variables
    """

    CI_ENV_VARIABLE = "TEAMCITY_VERSION"
    CI_TYPE = CiType.TEAM_CITY
    ENV_VARIABLE_PREFIXES = [
        "BUILD_",
        "HOSTNAME",
        "SERVER_URL",
        "TEAMCITY_",
    ]

    @classmethod
    def resolve(cls) -> Optional[Dict[str, str]]:
        """
        Returns a dictionary with Jenkins environment variable if 'cls.TEAMCITY_VERSION' exists
        in operating system environment. Otherwise returns None.

        Args:
            cls (JenkinsContextLoader):

        Returns:
            Union(Dict[str, str], optional): Team City environment variables in key-value format or None if
            'cls.TEAMCITY_VERSION' does not exists in os.environment.
        """
        if cls.CI_ENV_VARIABLE in os.environ:
            return cls.load_context_variables(cls.ENV_VARIABLE_PREFIXES)
        else:
            return None


class CircleCiContextResolver(BaseContextLoader):
    """
    A class that inherit from BaseContextLoader,used to represent TeamCityCIContext with its own environment variables.

    Attributes:
        CI_ENV_VARIABLE (str): 'CIRCLECI'
        CI_TYPE (CiType):
        ENV_VARIABLE_PREFIXES (List[str]): List of prefixes to find and load Circle CI environment variables.
    """

    CI_ENV_VARIABLE = "CIRCLECI"
    CI_TYPE = CiType.CIRCLE_CI
    ENV_VARIABLE_PREFIXES = ["CIRCLE", "HOSTNAME"]

    @classmethod
    def resolve(cls) -> Optional[Dict[str, str]]:
        """
        Returns a dictionary with CIRCLECI environment variable if 'cls.CI_ENV_VARIABLE' exists
        in operating system environment. Otherwise returns None.

        Args:
            cls (JenkinsContextLoader):

        Returns:
            Union(Dict[str, str], optional): CIRCLECI environment variables in key-value format or None if
            'cls.CI_ENV_VARIABLE' does not exists in os.environment.
        """
        if cls.CI_ENV_VARIABLE in os.environ:
            return cls.load_context_variables(cls.ENV_VARIABLE_PREFIXES)
        else:
            return None


class TravisCiContextResolver(BaseContextLoader):
    """
    A class that inherit from BaseContextLoader,used to represent TravisCiContext with its own environment variables.

    Attributes:
        CI_ENV_VARIABLE (str): 'TRAVIS'
        CI_TYPE (CiType):
        ENV_VARIABLE_PREFIXES (List[str]): List of prefixes to find and load TRAVIS environment variables.
    """

    CI_ENV_VARIABLE = "TRAVIS"
    CI_TYPE = CiType.TRAVIS_CI
    ENV_VARIABLE_PREFIXES = ["TRAVIS", "USER"]

    @classmethod
    def resolve(cls) -> Optional[Dict[str, str]]:
        """
        Returns a dictionary with TRAVIS environment variable if 'cls.CI_ENV_VARIABLE' exists
        in operating system environment. Otherwise returns None.

        Args:
            cls (JenkinsContextLoader):

        Returns:
            Union(Dict[str, str], optional): TRAVIS environment variables in key-value format or None if
            'cls.CI_ENV_VARIABLE' does not exists in os.environment.
        """
        if cls.CI_ENV_VARIABLE in os.environ:
            return cls.load_context_variables(cls.ENV_VARIABLE_PREFIXES)
        else:
            return None


def resolve_ci_context() -> Optional[CiContextModel]:
    ci_tools: List[Type[BaseContextLoader]] = [
        JenkinsContextLoader,
        TeamCityCiContextResolver,
        CircleCiContextResolver,
        TravisCiContextResolver,
    ]

    ci_context: Optional[Type] = None
    for resolver in ci_tools:
        env_variables = resolver.resolve()  # type: ignore
        if env_variables:
            ci_context = resolver
            break

    if ci_context:
        return CiContextModel(ci_type=ci_context.CI_TYPE.value, env_variables=ci_context.resolve())
    else:
        return None
